package org.jboss.cache.loader;

import org.jboss.cache.config.CacheLoaderConfig.IndividualCacheLoaderConfig;

import java.util.Properties;

/**
 * AdjListJDBCCacheLoaderConfig
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani (manik@jboss.org)</a>
 * @author <a href="mailto:galder.zamarreno@jboss.com">Galder Zamarreno</a>
 */
public class AdjListJDBCCacheLoaderConfig extends IndividualCacheLoaderConfig
{
    /**
     * The serialVersionUID
     */
    private static final long serialVersionUID = -8371846151643130281L;

    private boolean createTable;
    private String createTableDDL;
    private String datasourceName;
    private String deleteAllSql;
    private String deleteNodeSql;
    private boolean dropTable;
    private String dropTableDDL;
    private String driverClass;
    private String insertNodeSql;
    private String jdbcURL;
    private String jdbcUser;
    private String jdbcPassword;
    private String selectChildFqnsSql;
    private String selectChildNamesSql;
    private String selectNodeSql;
    private String updateNodeSql;
    private String updateTableSql;
    private String connectionFactoryClass;
    protected String table;
    protected String nodeColumn;
    protected String fqnColumn;

    public AdjListJDBCCacheLoaderConfig()
    {
    }

    /**
     * For use by {@link JDBCCacheLoaderOld}.
     *
     * @param base generic config object created by XML parsing.
     */
    AdjListJDBCCacheLoaderConfig(IndividualCacheLoaderConfig base)
    {
        populateFromBaseConfig(base);
    }

    public boolean getCreateTable()
    {
        return createTable;
    }

    public void setCreateTable(boolean createTable)
    {
        testImmutability("createTable");
        this.createTable = createTable;
    }

    public String getCreateTableDDL()
    {
        return createTableDDL;
    }

    public void setCreateTableDDL(String createTableDDL)
    {
        testImmutability("createTableDDL");
        this.createTableDDL = createTableDDL;
    }

    public String getDatasourceName()
    {
        return datasourceName;
    }

    public void setDatasourceName(String datasourceName)
    {
        testImmutability("datasourceName");
        this.datasourceName = datasourceName;
    }

    public String getDeleteAllSql()
    {
        return deleteAllSql;
    }

    public void setDeleteAllSql(String deleteAllSql)
    {
        testImmutability("deleteAllSql");
        this.deleteAllSql = deleteAllSql;
    }

    public String getDeleteNodeSql()
    {
        return deleteNodeSql;
    }

    public void setDeleteNodeSql(String deleteNodeSql)
    {
        testImmutability("deleteNodeSql");
        this.deleteNodeSql = deleteNodeSql;
    }

    public String getDriverClass()
    {
        return driverClass;
    }

    public void setDriverClass(String driverClass)
    {
        testImmutability("driverClass");
        this.driverClass = driverClass;
    }

    public boolean getDropTable()
    {
        return dropTable;
    }

    public void setDropTable(boolean dropTable)
    {
        testImmutability("dropTable");
        this.dropTable = dropTable;
    }

    public String getInsertNodeSql()
    {
        return insertNodeSql;
    }

    public void setInsertNodeSql(String insertNodeSql)
    {
        testImmutability("insertNodeSql");
        this.insertNodeSql = insertNodeSql;
    }

    public String getSelectChildFqnsSql()
    {
        return selectChildFqnsSql;
    }

    public void setSelectChildFqnsSql(String selectChildFqnsSql)
    {
        testImmutability("selectChildFqnsSql");
        this.selectChildFqnsSql = selectChildFqnsSql;
    }

    public String getSelectNodeSql()
    {
        return selectNodeSql;
    }

    public void setSelectNodeSql(String selectNodeSql)
    {
        testImmutability("selectNodeSql");
        this.selectNodeSql = selectNodeSql;
    }

    public String getTable()
    {
        return table;
    }

    public void setTable(String table)
    {
        testImmutability("table");
        this.table = table;
    }

    public String getUpdateTableSql()
    {
        return updateTableSql;
    }

    public void setUpdateTableSql(String updateTableSql)
    {
        testImmutability("updateTableSql");
        this.updateTableSql = updateTableSql;
    }

    public String getDropTableDDL()
    {
        return dropTableDDL;
    }

    public void setDropTableDDL(String dropTableDDL)
    {
        testImmutability("dropTableDDL");
        this.dropTableDDL = dropTableDDL;
    }

    public String getSelectChildNamesSql()
    {
        return selectChildNamesSql;
    }

    public void setSelectChildNamesSql(String selectChildNamesSql)
    {
        testImmutability("selectChildNamesSql");
        this.selectChildNamesSql = selectChildNamesSql;
    }

    public String getUpdateNodeSql()
    {
        return updateNodeSql;
    }

    public void setUpdateNodeSql(String updateNodeSql)
    {
        testImmutability("updateNodeSql");
        this.updateNodeSql = updateNodeSql;
    }

    public String getJdbcPassword()
    {
        return jdbcPassword;
    }

    public void setJdbcPassword(String jdbcPassword)
    {
        testImmutability("jdbcPassword");
        this.jdbcPassword = jdbcPassword;
    }

    public String getJdbcURL()
    {
        return jdbcURL;
    }

    public void setJdbcURL(String jdbcURL)
    {
        testImmutability("jdbcURL");
        this.jdbcURL = jdbcURL;
    }

    public String getJdbcUser()
    {
        return jdbcUser;
    }

    public void setJdbcUser(String jdbcUser)
    {
        testImmutability("jdbcUser");
        this.jdbcUser = jdbcUser;
    }

    public String getConnectionFactoryClass()
    {
        return connectionFactoryClass;
    }

    public void setConnectionFactoryClass(String connectionFactoryClass)
    {
        testImmutability("connectionFactoryClass");
        this.connectionFactoryClass = connectionFactoryClass;
    }

    public void setProperties(Properties props)
    {
        super.setProperties(props);
        datasourceName = props.getProperty("cache.jdbc.datasource");
        if (datasourceName == null)
        {
            this.driverClass = AdjListJDBCCacheLoader.getRequiredProperty(props, "cache.jdbc.driver");
            this.jdbcURL = AdjListJDBCCacheLoader.getRequiredProperty(props, "cache.jdbc.url");
            this.jdbcUser = AdjListJDBCCacheLoader.getRequiredProperty(props, "cache.jdbc.user");
            this.jdbcPassword = AdjListJDBCCacheLoader.getRequiredProperty(props, "cache.jdbc.password");

            if (log.isDebugEnabled())
            {
                log.debug("Properties: " +
                        "cache.jdbc.url=" +
                        jdbcURL +
                        ", cache.jdbc.driver=" +
                        driverClass +
                        ", cache.jdbc.user=" +
                        jdbcUser +
                        ", cache.jdbc.password=" +
                        jdbcPassword +
                        ", cache.jdbc.table=" + table);
            }
        }

        String prop = props.getProperty("cache.jdbc.table.create");
        this.createTable = (prop == null || Boolean.valueOf(prop));
        prop = props.getProperty("cache.jdbc.table.drop");
        this.dropTable = prop == null ? false : Boolean.valueOf(prop);

        this.table = props.getProperty("cache.jdbc.table.name", "jbosscache");
        String primaryKey = props.getProperty("cache.jdbc.table.primarykey", "jbosscache_pk");
        fqnColumn = props.getProperty("cache.jdbc.fqn.column", "fqn");
        String fqnType = props.getProperty("cache.jdbc.fqn.type", "varchar(255)");
        nodeColumn = props.getProperty("cache.jdbc.node.column", "node");
        String nodeType = props.getProperty("cache.jdbc.node.type", "blob");
        String parentColumn = props.getProperty("cache.jdbc.parent.column", "parent");

        selectChildNamesSql = "select " + fqnColumn + " from " + table + " where " + parentColumn + "=?";
        deleteNodeSql = "delete from " + table + " where " + fqnColumn + "=?";
        deleteAllSql = "delete from " + table;
        selectChildFqnsSql = "select " + fqnColumn + " from " + table + " where " + parentColumn + "=?";
        insertNodeSql = "insert into " +
                table +
                " (" +
                fqnColumn +
                ", " +
                nodeColumn +
                ", " +
                parentColumn +
                ") values (?, ?, ?)";
        updateNodeSql = "update " + table + " set " + nodeColumn + "=? where " + fqnColumn + "=?";
        selectNodeSql = "select " + nodeColumn + " from " + table + " where " + fqnColumn + "=?";

        createTableDDL = "create table " +
                table +
                "(" +
                fqnColumn +
                " " +
                fqnType +
                " not null, " +
                nodeColumn +
                " " +
                nodeType +
                ", " +
                parentColumn +
                " " +
                fqnType +
                ", constraint " + primaryKey + " primary key (" + fqnColumn + "))";

        dropTableDDL = "drop table " + table;
        connectionFactoryClass = props.getProperty("cache.jdbc.connection.factory", "org.jboss.cache.loader.NonManagedConnectionFactory");
    }

    public boolean equals(Object obj)
    {
        if (obj instanceof AdjListJDBCCacheLoaderConfig && equalsExcludingProperties(obj))
        {
            AdjListJDBCCacheLoaderConfig other = (AdjListJDBCCacheLoaderConfig) obj;

            return (this.createTable == other.createTable)
                    && safeEquals(createTableDDL, other.createTableDDL)
                    && safeEquals(datasourceName, other.datasourceName)
                    && safeEquals(deleteAllSql, other.deleteAllSql)
                    && safeEquals(deleteNodeSql, other.deleteNodeSql)
                    && safeEquals(driverClass, other.driverClass)
                    && (dropTable == other.dropTable)
                    && safeEquals(dropTableDDL, other.dropTableDDL)
                    && safeEquals(insertNodeSql, other.insertNodeSql)
                    && safeEquals(jdbcPassword, other.jdbcPassword)
                    && safeEquals(jdbcURL, other.jdbcURL)
                    && safeEquals(jdbcUser, other.jdbcUser)
                    && safeEquals(selectChildFqnsSql, other.selectChildFqnsSql)
                    && safeEquals(selectChildNamesSql, other.selectChildNamesSql)
                    && safeEquals(selectNodeSql, other.selectNodeSql)
                    && safeEquals(table, other.table)
                    && safeEquals(updateNodeSql, other.updateNodeSql)
                    && safeEquals(updateTableSql, other.updateTableSql)
                    && safeEquals(connectionFactoryClass, other.connectionFactoryClass);
        }

        return false;
    }

    public int hashCode()
    {
        int result = hashCodeExcludingProperties();
        result = 31 * result + (createTable ? 0 : 1);
        result = 31 * result + (createTableDDL == null ? 0 : createTableDDL.hashCode());
        result = 31 * result + (datasourceName == null ? 0 : datasourceName.hashCode());
        result = 31 * result + (deleteAllSql == null ? 0 : deleteAllSql.hashCode());
        result = 31 * result + (deleteNodeSql == null ? 0 : deleteNodeSql.hashCode());
        result = 31 * result + (driverClass == null ? 0 : driverClass.hashCode());
        result = 31 * result + (dropTable ? 0 : 1);
        result = 31 * result + (dropTableDDL == null ? 0 : dropTableDDL.hashCode());
        result = 31 * result + (insertNodeSql == null ? 0 : insertNodeSql.hashCode());
        result = 31 * result + (jdbcPassword == null ? 0 : jdbcPassword.hashCode());
        result = 31 * result + (jdbcUser == null ? 0 : jdbcUser.hashCode());
        result = 31 * result + (jdbcURL == null ? 0 : jdbcURL.hashCode());
        result = 31 * result + (selectChildFqnsSql == null ? 0 : selectChildFqnsSql.hashCode());
        result = 31 * result + (selectChildNamesSql == null ? 0 : selectChildNamesSql.hashCode());
        result = 31 * result + (selectNodeSql == null ? 0 : selectNodeSql.hashCode());
        result = 31 * result + (table == null ? 0 : table.hashCode());
        result = 31 * result + (updateNodeSql == null ? 0 : updateNodeSql.hashCode());
        result = 31 * result + (updateTableSql == null ? 0 : updateTableSql.hashCode());
        result = 31 * result + (connectionFactoryClass == null ? 0 : connectionFactoryClass.hashCode());

        return result;
    }

   @Override
   public AdjListJDBCCacheLoaderConfig clone() throws CloneNotSupportedException
   {
      return (AdjListJDBCCacheLoaderConfig) super.clone();
   }

}