package org.jboss.cache.optimistic;

import org.jboss.cache.InvocationContext;
import org.jboss.cache.interceptors.Interceptor;
import org.jboss.cache.marshall.MethodCall;
import org.jboss.cache.marshall.MethodDeclarations;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

/**
 * Handles putXXX() methods: if the given node doesn't exist, it will be created
 * (depending on the create_if_not_exists argument)
 *
 * @author Bela Ban
 * @version $Id: CreateIfNotExistsInterceptor.java,v 1.7 2005/01/26 11:45:14
 *          belaban Exp $
 */
public class MockInterceptor extends Interceptor
{
   private Method called = null;

   private List<Method> calledlist = new ArrayList<Method>();
   private List<Integer> calledIdsList = new ArrayList<Integer>();

   @Override
   public synchronized Object invoke(InvocationContext ctx) throws Throwable
   {
      MethodCall m = ctx.getMethodCall();
      if (!MethodDeclarations.isBlockUnblockMethod(m.getMethodId()))
      {
         calledlist.add(m.getMethod());
         calledIdsList.add(m.getMethodId());
         called = m.getMethod();
      }

      return null;
   }

   /**
    * @return Returns the called.
    */
   public Method getCalled()
   {
      return called;
   }

   public List<Method> getAllCalled()
   {
      return calledlist;
   }

   public List<Integer> getAllCalledIds()
   {
      return calledIdsList;
   }

   /**
    * @param called The called to set.
    */
   public void setCalled(Method called)
   {
      this.called = called;
   }
}